/*
 * newsmack label [ command [ arg ] ... ]
 *
 * Port to busybox: Casey Schaufler <casey@schaufler-ca.com>
 *
 * Copyright (C) 2007 Casey Schaufler <casey@schaufler-ca.com>
 *
 */

#include "libbb.h"

int newsmack_main(int argc, char **argv);
int newsmack_main(int argc, char **argv)
{
	struct passwd *pwd;
	char *newargv[256]; /* yes, I know */
	char *smack;
	int i;
	int newargc;

	if (argc <= 1)
		bb_error_msg_and_die("No smack value specified");

	smack = argv[1];

	/*
	 * Start a shell if no command is specified
	 */
	if (argc == 2) {
		fprintf(stderr, "%s: start a shell at \"%s\"\n",
			argv[0], smack);
		newargv[0] = strdup("sh");
		newargv[1] = NULL;
	} else {
		for (newargc = 0, i = 2; i < argc; newargc++, i++)
			newargv[newargc] = argv[i];
		newargv[newargc] = NULL;
	}

	/*
	 * Verify the user is allowed the Smack label
	 */
	pwd = getpwuid(getuid());
	if (pwd == NULL)
		bb_error_msg_and_die("User name not obtained");
	if (smack_user_allowed(pwd->pw_name, smack) != 0)
		bb_error_msg_and_die("User not allowed this smack label");
	/*
	 * Set the process label.
	 */
	i = smack_to_proc(-1, smack);
	if (i < 0)
		bb_error_msg_and_die("Cannot set smack");

	/*
	 * Do the exec
	 */
	execvp(newargv[0], newargv);

	bb_error_msg_and_die("%s: exec failure.", newargv[0]);
}
